set statistics io on;
set nocount on;

CREATE TABLE #Data
(
    Id int NOT NULL,
    Field1 varchar(50) NOT NULL,
    Field2 varchar(50) NULL,
    SomeNum1 int NOT NULL,
    SomeNum2 int NULL
);

INSERT INTO #Data(Id, Field1, Field2, SomeNum1, SomeNum2) VALUES
(1, 'a', 'b', 1, 2),
(2, 'a', NULL, 1, 2),
(3, 'b', 'a', 1, 2),
(4, 'b', 'a', 2, 1),
(5, 'b', NULL, 1, NULL);

/*
	Our error states are as follows:
	1)  Field2 is NULL
	2)  Field1 has a greater value than Field2
	3)  SomeNum2 is NULL
	4)  SomeNum1 is larger than SomeNum2
*/

/* The non-tally solution is fine, but includes 4 table scans. */
SELECT Id, 'Field2 is NULL!' as ErrorMessage FROM #Data WHERE Field2 IS NULL
UNION ALL
SELECT Id, 'SomeNum2 is NULL!' FROM #Data WHERE SomeNum2 IS NULL
UNION ALL
SELECT Id, 'Field1 is bigger than Field2!' FROM #Data WHERE Field1 > Field2
UNION ALL
SELECT Id, 'SomeNum1 is bigger than SomeNum2!' FROM #Data WHERE SomeNum1 > SomeNum2
ORDER BY Id, ErrorMessage;

/* The tally table solution only reads through our data file once, making it scale better. */
WITH tally as
(
    SELECT 1 as N, 'Field2 is NULL!' as ErrorMessage
    UNION ALL
    SELECT 2 as N, 'SomeNum2 is NULL!' as ErrorMessage
    UNION ALL
    SELECT 3 as N, 'Field1 is bigger than Field2!' as ErrorMessage
    UNION ALL
    SELECT 4 as N, 'SomeNum1 is bigger than SomeNum2!' as ErrorMessage
)
SELECT
    d.Id,
    t.ErrorMessage
FROM
    #Data d
    CROSS JOIN tally t
WHERE
    (
        (t.N = 1 AND d.Field2 IS NULL)
        OR (t.N = 2 AND d.SomeNum2 IS NULL)
        OR (t.N = 3 AND d.Field1 > d.Field2)
        OR (t.N = 4 AND d.SomeNum1 > d.SomeNum2)
    );