SET NOCOUNT ON;

DECLARE @Parameter VARCHAR(8000) = 'Part#1,PartName1,1|Part#2,PartName2,2|Part#3,PartName3,3';

if exists(select * from sys.tables where name = 'Elements' and schema_name(schema_id) = 'dbo') drop table dbo.Elements;
Create Table Elements
(
    ElementNumber INT IDENTITY(1,1), --Order it appears in original string
    ElementValue  VARCHAR(8000),     --The string value of the element
    RowNum        INT,               --One of the two indices
    ColNum        INT                --One of the two indices
)
 
DECLARE @GroupCount INT; 
--===== Determine the number of elements in a group (assumes they're all the same)
 SELECT @GroupCount =
        -- Find the length of the first group...
        LEN(SUBSTRING(@Parameter,1,CHARINDEX('|',@Parameter)))
        -- ... subtract the length of the first group without any commas...
      - LEN(REPLACE(SUBSTRING(@Parameter,1,CHARINDEX('|',@Parameter)),',',''))
        -- ... and add 1 because there is always 1 element more than commas.
      + 1;
 
SET @Parameter = ',' + REPLACE(@Parameter,'|',',') + ',';
 
--===== Join the Tally table to the string at the character level and
     -- when we find a comma, insert what's between that comma and
     -- the next comma into the Elements table
INSERT INTO Elements (ElementValue)
SELECT
	SUBSTRING(@Parameter, N + 1, CHARINDEX(',',@Parameter, N + 1) - N - 1)
FROM 
	dbo.Tally
WHERE 
	N < LEN(@Parameter)
	AND SUBSTRING(@Parameter, N, 1) = ',';
 
--===== Calculate and update the "row" and "column" indices
 UPDATE Elements
 SET 
	RowNum = (ElementNumber-1) / @GroupCount, --"Zero" based row
	ColNum = (ElementNumber-1) % @GroupCount; --"Zero" based col

declare @columns table
(
    ColumnNumber int,
    Name nvarchar(30)
);

/* To get real dynamic columns, you could create a column table and include these records.
    I'm using a table variable for the sake of simplicity. */
insert into @columns(ColumnNumber, Name) values
(0, 'PartNumber'), (1, 'PartName'), (2, 'QuantityOrdered');
 
declare @sql nvarchar(max);
set @sql = 'select ';
 
with records(ColumnNumber, ColumnText) as
(
    select
            e.ColNum,
            'max(case when colnum = '
            + cast(e.ColNum as nvarchar(2))
            + ' then ElementValue end) as '
            + c.Name
    from
        Elements e
        inner join @columns c on e.ColNum = c.ColumnNumber
    where
        e.RowNum = 0
),
unpivotedrecords as
(
    select distinct
        RecordText = STUFF
					(
	                    (
							select ( ', ' + ColumnText )
							from records r2
							order by ColumnNumber
							for xml path ( '' )
						), 1, 1, ''
					)
    from
        records r1
)
select
    @sql = @sql + RecordText
from
    unpivotedrecords;
 
set @sql = @sql + ' from Elements group by RowNum;';
 
--select @sql;
exec sp_executesql @sql;